<?php

namespace app\admin\controller\member;

use app\admin\validate\member\RuleAddedValidate;
use app\admin\validate\member\RulePricingValidate;
use app\controller\Curd;
use app\model\Member;
use app\model\MemberQuota;
use app\model\RuleAdded;
use app\model\RulePricing;
use app\model\SysSerial;
use support\Redis;
use support\exception\BusinessException;
use support\Request;
use support\Response;

class QuotaController extends Curd
{
    public function __construct()
    {
        $this->model = new MemberQuota();
//        $this->validate = true;
//        $this->validateClass = new RuleAddedValidate();
    }

    public function select(Request $request): Response
    {
        $keywords = $request->get('keywords', '');
        $memberIds = [];
        if (!empty($keywords)) {
            $memberIds = Member::leftJoin('member_cert', 'member_cert.join_cert_member_id', '=', 'member.member_id')
                ->where('member.member_mobile', 'like', '%' . $keywords . '%')
                ->orWhere('member_cert.member_cert_name', 'like', '%' . $keywords . '%')
                ->pluck('member.member_id')->toArray();
            if (empty($memberIds)) {
                return json_success('', []);
            }
        }

        [$where, $format, $limit, $field, $order] = $this->selectInput($request);
        if (!empty($memberIds)) {
            $where['join_quota_member_id'] = ['in', $memberIds];
        }
        $order = $request->get('order', 'desc');
        $field = $field ?? 'member_quota_addtimes';
        $query = $this->doSelect($where, $field, $order);
        return $this->doFormat($query, $format, $limit);
    }

    protected function doSelect(array $where, string $field = null, string $order = 'desc')
    {
        $model = $this->model->with([
            'member' => function ($query) {
                $query->select('member_id', 'member_mobile');
            },
            'cert' => function ($query) {
                $query->select('join_cert_member_id', 'member_cert_name');
            },
            'component' => function ($query) {
                $query->select('rule_added_component_id', 'rule_added_component_name');
            }
        ]);
        foreach ($where as $column => $value) {
            if (is_array($value)) {
                if ($value[0] === 'like' || $value[0] === 'not like') {
                    $model = $model->where($column, $value[0], "%$value[1]%");
                } elseif (in_array($value[0], ['>', '=', '<', '<>'])) {
                    $model = $model->where($column, $value[0], $value[1]);
                } elseif ($value[0] == 'in' && !empty($value[1])) {
                    $valArr = $value[1];
                    if (is_string($value[1])) {
                        $valArr = explode(",", trim($value[1]));
                    }
                    $model = $model->whereIn($column, $valArr);
                } elseif ($value[0] == 'not in' && !empty($value[1])) {
                    $valArr = $value[1];
                    if (is_string($value[1])) {
                        $valArr = explode(",", trim($value[1]));
                    }
                    $model = $model->whereNotIn($column, $valArr);
                } elseif ($value[0] == 'null') {
                    $model = $model->whereNull($column);
                } elseif ($value[0] == 'not null') {
                    $model = $model->whereNotNull($column);
                } elseif ($value[0] !== '' || $value[1] !== '') {
                    $model = $model->whereBetween($column, $value);
                }
            } else {
                $model = $model->where($column, $value);
            }
        }
        if ($field) {
            $model = $model->orderBy($field, $order);
        }
        return $model;
    }

//    protected function insertInput(Request $request): array
//    {
//        $data = $this->inputFilter($request->post());
//        $data['rule_added_id'] = "RA" . str_pad(SysSerial::getSerial(), 16, '0', STR_PAD_LEFT) . random_string(6, 'up');
//
//        return $data;
//    }

    protected function updateInput(Request $request): array
    {
        $primary_key = $this->model->getKeyName();
        $id = $request->post($primary_key);
        $data = $this->inputFilter($request->post());
//        if (!empty($data['rule_pricing_goods_json'])) {
//            $data['rule_pricing_goods_json'] = json_encode(explode(',', $data['rule_pricing_goods_json']));
//        } else {
//            $data['rule_pricing_goods_json'] = '[]';
//        }
        $model = $this->model->find($id);
        if (!$model) {
            throw new BusinessException('记录不存在', 2);
        }
        unset($data[$primary_key]);
        return [$id, $data];
    }

    /**
     * @Desc 额度核销
     * @Author Gorden
     * @Date 2024/6/4 8:32
     *
     * @param Request $request
     * @return Response
     */
    public function writeOff(Request $request)
    {
        $quotaIds = $request->post('quota_id', []);
        $code = $request->post('sms_code', '');

        $quotas = MemberQuota::whereIn('member_quota_id', $quotaIds)->get()->toArray();
        $memberIds = array_column($quotas, 'join_quota_member_id');

        if (count(array_unique($memberIds)) > 1) {
            return json_fail("不可同时核销多个会员的权益");
        }

        $status = array_column($quotas, 'member_quota_status');
        if (in_array('USED', $status) || in_array('CANCEL',$status)) {
            return json_fail('存在已核销 / 无效的权益，请重新选择');
        }

        $member = Member::find(current($memberIds));
        $mobile = $member->member_mobile;
        $key = "SMS:CODE:QUOTA:" . $mobile;
        $redisCode = Redis::get($key);
        if ($redisCode != $code) {
            return json_fail("验证码错误，请重新输入");
        }

        try {
            MemberQuota::whereIn('member_quota_id', $quotaIds)->update(['member_quota_status' => 'USED']);
        } catch (\Exception $e) {
            return json_fail("核销失败");
        }

        return json_success("核销成功");
    }

}